@@ -1,5 +1,5 @@
 /*  GKrellMMS: GKrellM XMMS Plugin
- |  Copyright (C) 2000-2002 Sander Klein Lebbink
+ |  Copyright (C) 2000-2004 Sander Klein Lebbink
  |
  |  Original Author:  Sander Klein Lebbink <sander@cerberus.demon.nl>
  |  Current Maintainer: Sjoerd Simons <sjoerd@luon.net>
@@ -47,9 +47,7 @@ static ControlButton prev_button,
                      eject_button;
 
 /* Define xmms-vars */
-static gint cur_len;
 static gint where_to_jump;
-static gint t;
 
 static gboolean xmms_playing = FALSE;
 static GkrellmMonitor *monitor;
@@ -72,10 +70,12 @@ static GkrellmPanel *control_panel;
 static GkrellmKrell *time_krell, *slider_in_motion;
 
 static GkrellmDecal *xmms_decal,*led_decal;
-static gint led_off_index,
-            led_running_index,
-            led_paused_index,
-            led_playing_index;
+static gint  led_off_index = 0,
+             led_running_index = 1,
+             led_paused_index = 2,
+             led_playing_index= 3;
+
+static gint	scroll_separator_len;
 
 static gboolean	gkrellmrc_button_placement;
 static gboolean got_motion;
@@ -114,13 +114,12 @@ void do_xmms_command(gint i) {
     case gkrellmms_paus: xmms_remote_pause(xmms_session); break;
     case gkrellmms_stop:
       xmms_remote_stop(xmms_session);
-      time_krell->previous =
-      t = 0;
+// FIXME      time_krell->previous = t = 0;
       break;
     case gkrellmms_next: xmms_remote_playlist_next(xmms_session); break;
     case gkrellmms_eject:
       if (eject_opens_playlist)
-        pl_open_func(NULL, NULL);
+        pl_show_playlist();
       else
         xmms_remote_eject(xmms_session);
       break;
@@ -145,98 +144,132 @@ set_panel_status(void) {
 	gkrellm_set_button_sensitive(eject_button.button, xmms_running);
 }
 
+void
+gkrellmms_set_scroll_separator_len(void)
+	{
+	scroll_separator_len = gkrellm_gdk_string_width(
+							scroll_text->text_style.font,
+							scroll_separator);
+	}
+
+
+static gchar *
+get_scrolling_title_text(gint *ret_width, gboolean reset)
+	{
+	gint          cur_time, cur_position;
+	gchar         *cur_title;
+	static gint   time, position, width;
+	static gchar	*title, *scrolling_text;
+
+	cur_time = pl_get_current_time();
+	cur_position = pl_get_current_position();
+	cur_title = pl_get_current_title();
+	if (   !scrolling_text || reset
+			|| cur_time != time || cur_position != position
+			|| gkrellm_dup_string(&title, cur_title)
+	   )
+		{
+		time = cur_time;
+		position = cur_position;
+		g_free(scrolling_text);
+		if (time > 0)
+			scrolling_text = g_strdup_printf("%d. %s (%d:%02d)",
+						position, title, time / (60 * 1000), time/1000 % 60);
+		else
+				scrolling_text = g_strdup_printf("%d. %s", position, title);
+		width = gkrellm_gdk_string_width(scroll_text->text_style.font,
+							scrolling_text);
+		}
+	if (ret_width)
+		*ret_width = width;
+	if (reset)
+		gkrellmms_set_scroll_separator_len();
+	return scrolling_text;
+	}
+
 static void
 update_gkrellmms() {
 	GkrellmMargin *margin;
-	static gint	output_time, len, w, separator_len;
-	static gint	prev_where;
+	static gint	output_time, len, w;
+	static gint	prev_position = -1;
 	gchar		*scrolling_title_text;
 	gchar		*prev_scrolling_tooltip_text;
 	gchar		*tooltip_utf8 = NULL, *tooltip_locale = NULL;
 	gchar		*more_scrolled;
-	gint		pos_time;
-	gchar		*filename;
 	gchar		*xmms_string;
-	playlist_def *templ;
 	gint		rate;
 	gint		freq;
 	gint		nch;
-	static gint	on_index, off_index, led_on_ticks;
+  gint    time;
+  gint    playlist_changed = FALSE;
+  gint    position_changed = FALSE;
+  gint slider_position = 0;
+	static gint	on_index = 0, off_index = 0, led_status = 0,led_on = FALSE;
 
-	if (pGK->second_tick) set_panel_status();
+	if (pGK->second_tick) 
+    set_panel_status();
 
-	if (xmms_remote_is_running(xmms_session)) {
-		xmms_running = TRUE;
+  playlist_changed = update_playlist();
+
+	if ((xmms_running = xmms_remote_is_running(xmms_session))) {
+    /* position is changed in the playlist when the playlist is changed too !
+     */
+    position_changed = 
+      pl_get_current_position() != prev_position || playlist_changed;
+    prev_position = pl_get_current_position();
 		xmms_playing = xmms_remote_is_playing(xmms_session);
 
-		prev_where = xmms_pos;
-		xmms_pos = xmms_remote_get_playlist_pos(xmms_session);
-		cur_len = xmms_remote_get_playlist_time(xmms_session, xmms_pos);
-
-		/* Check if there is a local playlist set; if not, load it!
-		*/
-		if (!plist || 
-        (g_list_length(plist) != xmms_remote_get_playlist_length(xmms_session))
-       )
-			update_playlist();
-
-		if (g_list_length(plist) > 0)
-			templ = (playlist_def *) g_list_nth(plist, xmms_pos)->data;
-		else {
-			templ = g_malloc(sizeof(playlist_def *));
-			templ->time = 0;
-			templ->title = "";
-			templ->filename = "";
-		}
-		pos_time = templ->time / 1000;
-
 		if (scroll_panel) {
-		  if ((prev_where != xmms_pos) || GK.second_tick) {
-				if (update_plist_statusbar(xmms_pos)) update_playlist_window();
-				update_plist_window_row(prev_where, xmms_pos);
-		  }
-
 			/* Scrollbar */
       if (scroll_enable && !scroll_in_motion) {
 				margin = gkrellm_get_style_margins(style);
 				w = gkrellm_chart_width() - margin->left - margin->right - 2;
 
-				if (templ->time > 0)
-					scrolling_title_text = g_strdup_printf("%d. %s (%d:%02d)",
-					xmms_pos + 1, templ->title, pos_time / 60, pos_time % 60);
-				else
-					scrolling_title_text = g_strdup_printf("%d. %s",
-				xmms_pos + 1, templ->title);
-
+				scrolling_title_text = get_scrolling_title_text(&len, FALSE);
+				time = pl_get_current_time();
 				if (scrolling_tooltip != NULL) {
 					xmms_remote_get_info(xmms_session, &rate, &freq, &nch);
-					prev_scrolling_tooltip_text = g_strdup(scrolling_tooltip_text);
-
-          g_free(scrolling_tooltip_text);
-
+					prev_scrolling_tooltip_text = scrolling_tooltip_text;
           scrolling_tooltip_text = g_strdup_printf("%s\n%d%s - %dHz - %s",
 				  scrolling_title_text, rate / 1000,
-				    	(templ->time  == -1) ? "bpm" : "kb/s", freq,
+				    	(time  == -1) ? "bpm" : "kb/s", freq,
 							(nch == 1) ? "mono" : "stereo");
 
-					if (strcmp(prev_scrolling_tooltip_text, scrolling_tooltip_text) != 0)
-						{ gkrellm_locale_dup_string(&tooltip_utf8,scrolling_tooltip_text,&tooltip_locale);
-						  gtk_tooltips_set_tip(scrolling_tooltip, scroll_panel->drawing_area,
+					if (!strcmp(prev_scrolling_tooltip_text, scrolling_tooltip_text)) { 
+            gkrellm_locale_dup_string(&tooltip_utf8,scrolling_tooltip_text,
+                                      &tooltip_locale);
+            gtk_tooltips_set_tip(scrolling_tooltip, scroll_panel->drawing_area,
 								                  tooltip_utf8, NULL);
-						  g_free(tooltip_utf8);
-						  g_free(tooltip_locale);
-						 }
+            g_free(tooltip_utf8);
+						g_free(tooltip_locale);
+					}
 					g_free(prev_scrolling_tooltip_text);
 			  }
-
-			  len = gdk_string_width(scroll_text->text_style.font, 
-           scrolling_title_text);
-			  separator_len = gdk_string_width(scroll_text->text_style.font,
-							scroll_separator);
+#if defined(GKRELLM_HAVE_DECAL_SCROLL_TEXT)
+				/* gkrellm >= 2.2.0 has a scroll text decal which minimizes slow
+				|  Pango renders as the decal text is scrolled (offset is changed).
+				*/
+				if (len > w) {
+					more_scrolled = g_strdup_printf("%s%s",
+							scrolling_title_text, scroll_separator);
+			  	x_scroll = (x_scroll + 1) % (len + scroll_separator_len);
+					gkrellm_decal_scroll_text_set_text(scroll_panel,
+									scroll_text, more_scrolled);
+					gkrellm_decal_scroll_text_horizontal_loop(scroll_text, TRUE);
+					gkrellm_decal_text_set_offset(scroll_text, -x_scroll, 0);
+		    }
+				else {
+					more_scrolled = g_strdup("");
+					x_scroll = 0;
+					gkrellm_decal_text_set_offset(scroll_text, 0, 0);
+					gkrellm_draw_decal_text(scroll_panel, scroll_text,
+									scrolling_title_text, 0);
+				}
+#else
 			  if (len > w) {
 			  	more_scrolled = g_strdup_printf("%s%s%s",
 			  	scrolling_title_text, scroll_separator, scrolling_title_text);
-			  	x_scroll = (x_scroll + 1) % (len + separator_len);
+			  	x_scroll = (x_scroll + 1) % (len + scroll_separator_len);
 			  	scroll_text->x_off = w - x_scroll - len;
 		    	} else {
 			    	more_scrolled = g_strdup("");
@@ -246,43 +279,16 @@ update_gkrellmms() {
 		  	gkrellm_draw_decal_text(scroll_panel, scroll_text,
 						(len > w) ? more_scrolled : scrolling_title_text,
 						(gulong) (len > w) ? (w - x_scroll - len) : -1);
+#endif
 		  	gkrellm_draw_panel_layers(scroll_panel);
 
-		  	g_free(scrolling_title_text);
 		  	g_free(more_scrolled);
 		  }
 	  }
 
-		/* Title-scroll is done, check for time-bar update */
-		if (pGK->second_tick) {
-			led_on_ticks = gkrellm_update_HZ() / 8 + 1;
-
-			if (g_list_length(plist)) {
-				/* Little after-check */
-				filename = xmms_remote_get_playlist_file(xmms_session, xmms_pos);
-				if (filename  == NULL || strcmp(templ->filename, filename) != 0) 
-          update_playlist();
-				g_free(filename);
-			}
-		}
-  
 		/* The time bar has a max.-scale of 100, so count the output-time
 		| to a max. of 100
 		*/
-		output_time = xmms_remote_get_output_time(xmms_session);
-		if (!slider_in_motion) {
-			t = cur_len ? ((output_time * 100) / cur_len) : 0;
-			if (t < 0)
-				t = 0;
-			else if (t > 100)
-				t = 100;
-	  }
-	} else {
-	  xmms_running = FALSE;
-		led_on_ticks = 0;
-		xmms_pos = -1;
-		t = 0;
-		if (plist) update_playlist();
 	}
 
 	/* Also draw xmms-status when xmms isn't running,
@@ -290,46 +296,58 @@ update_gkrellmms() {
 	*/
 	if (slider_in_motion == NULL) {
 		xmms_decal->x_off = 1;
-		if(xmms_running && (xmms_playing || (prev_where != xmms_pos))&& draw_time){ 
-      /* Have to re-define templ, or else gcc cries. :( */
-			if (g_list_length(plist) > 0)
-				templ = (playlist_def *) g_list_nth(plist, xmms_pos)->data;
-			else {
-				templ = g_malloc(sizeof(playlist_def *));
-				templ->time = 0;
-				templ->title = "";
-				templ->filename = "";
-			}
+		if(xmms_running && (xmms_playing || position_changed) && draw_time){ 
+		  output_time = xmms_remote_get_output_time(xmms_session);
+      /* calculate slider position */
+  	  slider_position = pl_get_current_time() ? 
+                             ((output_time * 100) / pl_get_current_time()) : 0;
+      if (slider_position < 0)
+        slider_position = 0;
+      else if (slider_position > 100)
+        slider_position = 100;
 
-			if (time_format || templ->time <= 0)
+      /* render timer string */
+			if (time_format || pl_get_current_time() <= 0) {
 				xmms_string = g_strdup_printf("%02d:%02d",
 						output_time / 60000, (output_time / 1000) % 60);
-			else {
+      } else {
 				xmms_string = g_strdup_printf(
 						draw_minus ? "-%02d:%02d" : "%02d:%02d",
-						(cur_len - output_time) / 60000,
-						((cur_len - output_time) / 1000) % 60);
+						(pl_get_current_time() - output_time) / 60000,
+						((pl_get_current_time() - output_time) / 1000) % 60);
 				if (draw_minus)
 					xmms_decal->x_off = -1;
 				}
-			} else xmms_string = g_strdup(gkrellmms_label);
+			} else {
+        xmms_string = g_strdup(gkrellmms_label);
+      }
 
 		gkrellm_draw_decal_text(control_panel, xmms_decal, xmms_string, -1);
-		if (!xmms_running) off_index = led_off_index;
-		else if (xmms_remote_is_paused(xmms_session)) {
+
+    /* calculate if led should be on or off */
+    if (led_status < 0) {
+      /* want to blink for two seconds */
+      led_status = gkrellm_update_HZ() * 2;
+    }
+    led_on = (--led_status) > gkrellm_update_HZ();
+
+		if (!xmms_running)  {
+      on_index = led_playing_index;
+      off_index = led_off_index;
+    } else if (xmms_remote_is_paused(xmms_session)) {
 			on_index = led_off_index;	/* invert the duty cycle */
 			off_index = led_paused_index;
 		} else if (!xmms_playing) {
 			on_index = led_running_index;
 			off_index = led_off_index;
 		} else	/* Playing */ {
-			led_on_ticks = 1;		/* no blinking */
+			led_on = TRUE; /* no blinking */
 			on_index = led_playing_index;
 		}
 		gkrellm_draw_decal_pixmap(control_panel, led_decal,
-				(led_on_ticks-- > 0) ? on_index : off_index);
+				led_on ? on_index : off_index);
 
-		gkrellm_update_krell(control_panel, time_krell, (gulong) t);
+		gkrellm_update_krell(control_panel, time_krell, (gulong) slider_position);
 		gkrellm_draw_panel_layers(control_panel);
 		g_free(xmms_string);
   }
@@ -350,7 +368,7 @@ slider_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data) {
   gint   z;
   GdkModifierType state;
 
-  if ((slider_in_motion != NULL) && g_list_length(plist)) {
+  if (slider_in_motion != NULL) {
     /* Check if button is still pressed, in case missed button_release */
     state = ev->state;
     if (!(state & GDK_BUTTON1_MASK))
@@ -363,14 +381,14 @@ slider_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data) {
       z = 0;
     else if (z > 100)
       z = 100;
-    t = z;
-    where_to_jump = (z * cur_len) / 100;
-    if (where_to_jump >= cur_len) where_to_jump = cur_len - 1;
 
-    got_motion = TRUE;
     time_krell->previous = 0;
-    gkrellm_update_krell(control_panel, time_krell, (gulong) t);
+    gkrellm_update_krell(control_panel, time_krell, (gulong) z);
     gkrellm_draw_panel_layers(control_panel);
+
+    where_to_jump = (z * pl_get_current_time()) / 100;
+    if (where_to_jump >= pl_get_current_time()) where_to_jump = pl_get_current_time() - 1;
+    got_motion = TRUE;
   }
   return TRUE;
 }
@@ -391,10 +409,8 @@ panel_button_release(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
   gint z;
   gint timer;
   time_t lt;
-  playlist_def *templ;
 
-  if (slider_in_motion && g_list_length(plist))
-  {
+  if (slider_in_motion) {
     if (!got_motion) {
       /* Also jump to time if you've clicked once on the Krell */
       z = ev->x * time_krell->full_scale / (gkrellm_chart_width() - 1);
@@ -402,17 +418,17 @@ panel_button_release(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
         z = 0;
       else if (z > 100)
         z = 100;
-      t = z;
-      where_to_jump = (z * cur_len) / 100;
-      if (where_to_jump >= cur_len) where_to_jump = cur_len -1;
+      where_to_jump = (z * pl_get_current_time()) / 100;
+      if (where_to_jump >= pl_get_current_time()) 
+        where_to_jump = pl_get_current_time() -1;
+
+      time_krell->previous = 0;
+      gkrellm_update_krell(time_bar, time_krell, (gulong) z);
+      gkrellm_draw_panel_layers(time_bar);
     }
 
-    templ = (playlist_def *) g_list_nth(plist, xmms_pos)->data;
-    if (where_to_jump > templ->time) return FALSE;
+    if (where_to_jump > pl_get_current_time()) return FALSE;
 
-    time_krell->previous = 0;
-    gkrellm_update_krell(time_bar, time_krell, (gulong) t);
-    gkrellm_draw_panel_layers(time_bar);
     /*
      | Let gkrellm sleep for about 0 seconds when sending this commands
      | when xmms isn't playing, so that xmms received the functions for
@@ -442,9 +458,7 @@ panel_button_release(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
         != (where_to_jump / 1000) && ((time(&lt) - timer) < 10))
       usleep(0);
   }
-  else if((slider_in_motion != NULL) && 
-           (!g_list_length(plist) || !xmms_playing)
-         )
+  else if((slider_in_motion != NULL) && !xmms_playing)
     xmms_remote_play(xmms_session);
 
   slider_in_motion = NULL;
@@ -489,7 +503,7 @@ panel_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
 
 static void
 scroll_bar_press(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
-  if (g_list_length(plist) && (ev->button == 1)) {
+  if (ev->button == 1) {
     scroll_in_motion = scroll_text;
     scroll_motion_x = ev->x;
   }
@@ -504,9 +518,8 @@ scroll_bar_release(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 static void
 scroll_bar_motion(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
   GkrellmMargin *margin;
-  gint w, len, change, separator_len;
+  gint w, len, change;
   GdkModifierType state;
-  playlist_def *templ;
   gchar *scrolling_title_text;
   gchar *more_scrolled;
 
@@ -522,39 +535,38 @@ scroll_bar_motion(GtkWidget *widget, GdkEventButton *ev, gpointer data) {
     margin = gkrellm_get_style_margins(style);
     w = gkrellm_chart_width() - margin->left - margin->right - 2;
 
-    templ = (playlist_def *) g_list_nth(plist, xmms_pos)->data;
     change = scroll_motion_x - ev->x;
     scroll_motion_x = ev->x;
 
-    if (templ->time > 0)
-      scrolling_title_text = g_strdup_printf("%d. %s (%d:%02d)",
-        xmms_pos + 1, templ->title, (templ->time / 1000) / 60, (templ->time / 1000) % 60);
-    else
-      scrolling_title_text = g_strdup_printf("%d. %s",
-        xmms_pos + 1, templ->title);
-
-    len = gdk_string_width(ts_alt->font, scrolling_title_text);
-    separator_len = gdk_string_width(ts_alt->font, scroll_separator);
+		scrolling_title_text = get_scrolling_title_text(&len, FALSE);
 
     if (len <= w) {
       scroll_in_motion = NULL;
-      g_free(scrolling_title_text);
       return;
     }
 
-    x_scroll = (x_scroll + change) % (len + separator_len);
-    if (x_scroll < 0)
-      x_scroll = len + separator_len;
+		x_scroll = (x_scroll + change) % (len + scroll_separator_len);
+		if (x_scroll < 0)
+			x_scroll = len + scroll_separator_len;
 
+#if defined(GKRELLM_HAVE_DECAL_SCROLL_TEXT)
+    more_scrolled = g_strdup_printf("%s%s",
+						scrolling_title_text, scroll_separator);
+
+		gkrellm_decal_scroll_text_set_text(scroll_panel,
+						scroll_text, more_scrolled);
+		gkrellm_decal_scroll_text_horizontal_loop(scroll_text, TRUE);
+		gkrellm_decal_text_set_offset(scroll_text, -x_scroll, 0);
+#else
     more_scrolled = g_strdup_printf("%s%s%s",
       scrolling_title_text, scroll_separator, scrolling_title_text);
 
     scroll_text->x_off = w - x_scroll - len;
     gkrellm_draw_decal_text(scroll_panel, scroll_text, more_scrolled, 
         (gulong) w - x_scroll - len);
+#endif
 
     gkrellm_draw_panel_layers(scroll_panel);
-    g_free(scrolling_title_text);
     g_free(more_scrolled);
 
   }
@@ -671,8 +683,11 @@ decal_position(GkrellmDecal *decal, gchar *name, gboolean text)
 #if defined(GKRELLM_HAVE_THEME_SCALE)
 		y = y * gkrellm_get_theme_scale();
 #endif
+
+#if !GKRELLM_CHECK_VERSION(2,1,100)
 		if (text)
 			y -= decal->y_baseline;
+#endif
 		gkrellm_move_decal(control_panel, decal, x, y);
 		}
 	}
@@ -801,9 +816,11 @@ create_gkrellmms(GtkWidget *vbox, gint first_create) {
       xmms_remote_main_win_toggle(xmms_session, FALSE);
 		if (xmms_autostart && !xmms_running)  xmms_start_func();
      
-		update_playlist();
+    pl_init();
 		control_panel = gkrellm_panel_new0();
-	}
+	} else {
+    update_playlist();
+  }
 
 	style = gkrellm_meter_style(DEFAULT_STYLE);
 	if (scroll_style) g_free(scroll_style);
@@ -857,7 +874,7 @@ create_gkrellmms(GtkWidget *vbox, gint first_create) {
 	gkrellm_set_krell_full_scale(time_krell, 100, 1);
 
 	m = gkrellm_get_style_margins(style);
-	w = gdk_string_width(ts->font, "-000:00");
+	w = gkrellm_gdk_string_width(ts->font, "-000:00");
 	xmms_decal = gkrellm_create_decal_text(control_panel, (gchar *) "A0", ts,
 				style, -1, -1, w);
 	xmms_decal->x += m->left;
@@ -921,6 +938,7 @@ create_gkrellmms(GtkWidget *vbox, gint first_create) {
 	gkrellm_insert_decal(control_panel, led_decal, TRUE);
 	gkrellm_insert_decal(control_panel, xmms_decal, TRUE);	/* On top */
 
+	get_scrolling_title_text(NULL, TRUE);		/* Sync with theme changes */
 	gkrellm_draw_panel_layers(control_panel);
 
 	set_panel_status();
@@ -981,6 +999,11 @@ static GkrellmMonitor  plugin_mon  =
   NULL                     /* Path if a plugin, filled by GkrellM   */
 };
 
+GkrellmMonitor *
+gkrellmms_get_monitor(void) {
+	return monitor;
+}
+
 GkrellmMonitor *
 gkrellm_init_plugin(void) {
   gchar *tmp;
@@ -989,18 +1012,15 @@ gkrellm_init_plugin(void) {
    bind_textdomain_codeset(PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
 
-  /* Popup-menu set */
-  running_factory = options_menu_factory(1);
-  not_running_factory = options_menu_factory(0);
-  pl_factory = playlist_menu_factory();
+	/* Popup-menu set */
+	running_factory = options_menu_factory(1);
+	not_running_factory = options_menu_factory(0);
 
   /* Default settings */
   control_panel = NULL;
   xmms_running = FALSE;
-  xmms_pos = 0;
   x_scroll = 0;
   total_plist_time = 0;
-  plist = NULL;
   scrolling_tooltip = NULL;
 
   tmp = g_strdup_printf("%s/%s", gkrellm_homedir(), GKRELLM_DATA_DIR);
@@ -1011,7 +1031,7 @@ gkrellm_init_plugin(void) {
 
   g_free(tmp);
 
-  playlist_dir = g_strdup("/");
+  playlist_dir = g_strdup(gkrellm_homedir());
   files_directory = g_strdup("/");
   gkrellmms_label = g_strdup("xmms");
   xmms_session = 0;
