@@ -1,5 +1,7 @@
+#ifndef GKRELLMMS_H
+#define GKRELLMMS_H
 /*  GKrellMMS: GKrellM XMMS Plugin
- |  Copyright (C) 2000-2002 Sander Klein Lebbink
+ |  Copyright (C) 2000-2004 Sander Klein Lebbink
  |
  |  Original Author:  Sander Klein Lebbink <sander@cerberus.demon.nl>
  |  Current Maintainer: Sjoerd Simons <sjoerd@luon.net>
@@ -24,6 +26,7 @@
 #include <gkrellm2/gkrellm.h>
 #include <xmms/xmmsctrl.h>
 
+#include "playlist.h"
 /* You can change these ones, but at your own risk.
  | Changing the maxlen_scroll_separator shouldn't
  | cause any weird things (though the scroll-bar could
@@ -38,7 +41,7 @@
 /* It's fun to set these ones to other values. ;) */
 #define GKRELLMMS_VERSION_MAJOR 2
 #define GKRELLMMS_VERSION_MINOR 1
-#define GKRELLMMS_VERSION_REV   12
+#define GKRELLMMS_VERSION_REV   21
 
 #define DEFAULT_STYLE style_id
 #define STYLE_NAME    "gkrellmms"
@@ -73,6 +76,10 @@
 #define GKRELLMMS_PLACE (MON_APM)
 #endif
 
+#if !GKRELLM_CHECK_VERSION(2,1,100)
+#define gkrellm_gdk_string_width	gdk_string_width
+#endif
+
 /* Don't edit stuff yourself under here,
  | or you'll have to be a programmer/gkrellmms hacker!
  */
@@ -91,19 +98,10 @@
 #define gkrellmms_aot      12
 #define gkrellmms_prefs    13
 
-typedef struct
-{
-  gchar *filename;
-  gchar *title;
-  gint time;
-} playlist_def;
 
 /* Very global vars */
 GtkItemFactory *running_factory;
 GtkItemFactory *not_running_factory;
-GtkItemFactory *pl_factory;
-
-GList *plist;
 
 GkrellmPanel *time_bar;
 gboolean xmms_running;
@@ -137,6 +135,7 @@ gboolean pl_window_open;
 /* Functions in gkrellmms.c */
 void update_gkrellmms_config(gint);
 void do_xmms_command(gint);
+void gkrellmms_set_scroll_separator_len(void);
 
 /* Functions in options.c */
 GtkItemFactory *options_menu_factory(gint);
@@ -147,14 +146,6 @@ void load_gkrellmms_config(gchar *);
 void save_gkrellmms_config(FILE *);
 void xmms_start_func(void);
 
-/* Functions in playlist.c */
-void pl_open_func(GtkWidget *, gpointer);
-void open_playlist(const gchar *, gint);
-void set_playlist_position(gint);
-void pl_open_popup(GtkWidget *, gpointer);
-void set_time_position(gint);
-GtkItemFactory *playlist_menu_factory(void);
-int update_plist_statusbar(gint);
-void update_playlist(void);
-void update_playlist_window(void);
-void update_plist_window_row(gint, gint);
+/* Functions in gkrelmms.c */
+GkrellmMonitor * gkrellmms_get_monitor(void);
+#endif /* GKRELLMMS_H */
